<?php

namespace Drupal\privatemsg;

/**
 * The interface for the Private Message Service.
 */
interface PrivateMsgServiceInterface {

  /**
   * Get thread from message.
   */
  public function getThreadFromMessage(int $message_id, int $user_id);

  /**
   * Get unread threads count.
   */
  public function getUnreadThreadCount();

  /**
   * Retrieve a list of thread IDs for the user.
   */
  public function getThreadsForUser(int $user_id);

  /**
   * Get last thread group number.
   */
  public function getLastThreadGroupNumber();

  /**
   * Update the last access time of group of threads for the given user.
   */
  public function updateThreadGroupLastAccessTime(int $user_id, int $thread_group);

  /**
   * Get the last access time of group of threads for the given user.
   */
  public function getThreadGroupLastAccessTime(int $user_id, int $thread_group);

  /**
   * Mark thread group as unread.
   */
  public function markThreadGroupAsUnread(int $user_id, int $thread_group);

  /**
   * Check if user blocked by current user.
   */
  public function isUserBlocked(int $user_id_who, int $user_id_blocked);

  /**
   * Block user.
   */
  public function blockUser(int $user_id);

  /**
   * Unblock user.
   */
  public function unblockUser(int $user_id);

  /**
   * Check if user can be blocked.
   */
  public function canBeBlocked(int $user_id);

  /**
   * Get list of blocked users by user id.
   */
  public function getBlockedByUserId(int $user_id);

}
